#include <time.h>
#include <OgreWindowEventUtilities.h>
#include "FadeScreenEffect.h"

FadeScreenEffect::FadeScreenEffect(Ogre::Root* root, const Ogre::String& name, const Ogre::String& mat)
: mSpeed(0.007f), mTimeAfterFade(2000), mRoot(root), mAlpha(1.f), mInOut(1.f)
{
    Ogre::ResourcePtr resptr = Ogre::MaterialManager::getSingleton().getByName(mat);
    mMat = dynamic_cast<Ogre::Material*>(resptr.getPointer());
    mTextUS = mMat->getTechnique(0)->getPass(0)->getTextureUnitState(0);

    mOverlay = Ogre::OverlayManager::getSingleton().getByName(name);
    Ogre::OverlayContainer* element = mOverlay->getChild("FadeInOutPanel");
    element->setMaterialName(mat);
    mOverlay->show();
}

FadeScreenEffect::~FadeScreenEffect()
{
    Ogre::OverlayManager::getSingleton().destroy(mOverlay);
}

void FadeScreenEffect::fade()
{
    while(mAlpha > 0.f)
    {
        mAlpha -= mSpeed;
        if(mAlpha < 0.f) mAlpha = 0.f;
        mTextUS->setAlphaOperation(Ogre::LBX_MODULATE, Ogre::LBS_MANUAL, Ogre::LBS_TEXTURE, mAlpha);
        Ogre::WindowEventUtilities::messagePump();
        mRoot->renderOneFrame();
        csleep(10);
    }

    Ogre::Timer t;
    t.reset();
    while(t.getMilliseconds() < mTimeAfterFade)
    {
        Ogre::WindowEventUtilities::messagePump();
        mRoot->renderOneFrame();
        csleep(10);
    }

    while(mAlpha < 1.f)
    {
        mAlpha += mSpeed;
        if(mAlpha > 1.f) mAlpha = 1.f;
        mTextUS->setAlphaOperation(Ogre::LBX_MODULATE, Ogre::LBS_MANUAL, Ogre::LBS_TEXTURE, mAlpha);
        Ogre::WindowEventUtilities::messagePump();
        mRoot->renderOneFrame();
        csleep(10);
    }

}

bool FadeScreenEffect::fade1frame()
{
    mAlpha += mSpeed * mInOut;
    mTextUS->setAlphaOperation(Ogre::LBX_MODULATE, Ogre::LBS_MANUAL, Ogre::LBS_TEXTURE, mAlpha);
    if(mAlpha > 1.f || mAlpha < 0.f) return true;
    return false;
}

void FadeScreenEffect::csleep(int ms)
{
    #if WIN32
    #include <windows.h>
        Sleep(ms);
    #else
        usleep(ms * 1000);
    #endif
}

void FadeScreenEffect::hide()
{
    mOverlay->hide();
}
