#include "Loader.h"

using namespace edge;

Loader::Loader()
{

}

edge::ai::Path* Loader::loadPath(const Ogre::String& str)
{
    std::vector<edge::ai::Station>* sl = new std::vector<edge::ai::Station>;

    FILE* f = fopen(str.c_str(),"rb");
    int num = 0;
    fread(&num, sizeof(int), 1, f);
    //printf("CNT=%i\n", num);

    for(int i=0; i<num; i++)
    {
        btVector3 v;
        fread(&v[0], sizeof(float), 1, f);
        fread(&v[1], sizeof(float), 1, f);
        fread(&v[2], sizeof(float), 1, f);
        //printf("V=%f;%f;%f\n",v[0],v[1],v[2]);
        sl->push_back(edge::ai::Station(v));
    }
    fclose(f);

    edge::ai::Path* apath = new edge::ai::Path(*sl);

    return apath;
}

CVehicle* Loader::loadVehicle(edge::World* m_world, const Ogre::String& cfg_path, int& id, bool reflection)
{
    Ogre::ConfigFile cfg;
    std::vector<Ogre::Vector3> conn;
    std::vector<bool> front;
    Ogre::StringVector sv, svmat;

    cfg.load(cfg_path);

    Ogre::String visual = cfg.getSetting("visual", "mesh");
    Ogre::String col = cfg.getSetting("collision", "mesh");
    Ogre::String wheel = cfg.getSetting("wheel", "mesh");
    Ogre::Vector3 scaling = Ogre::StringConverter::parseVector3(cfg.getSetting("scaling", "mesh"));
    Ogre::Vector3 wheelscaling = Ogre::StringConverter::parseVector3(cfg.getSetting("wheelscaling", "mesh"));
    Ogre::Vector3 colscaling = Ogre::StringConverter::parseVector3(cfg.getSetting("colscaling", "mesh"));

    int wheelflip = Ogre::StringConverter::parseInt(cfg.getSetting("wheelflip", "special"));

    float maxspeed = Ogre::StringConverter::parseReal(cfg.getSetting("maxspeed", "power"));
    float speedinc = Ogre::StringConverter::parseReal(cfg.getSetting("acceleration", "power"));
    float brake = Ogre::StringConverter::parseReal(cfg.getSetting("brake", "power"));
    float handling = Ogre::StringConverter::parseReal(cfg.getSetting("handling", "power"));
    float steerinc = Ogre::StringConverter::parseReal(cfg.getSetting("steerinc", "power"));
    float maxsteer = Ogre::StringConverter::parseReal(cfg.getSetting("maxsteer", "power"));

    sv = cfg.getMultiSetting("conn", "connection");
    int numwheels = sv.size();
    for(int i=0; i<numwheels; i++)
    {
        Ogre::Vector3 v = Ogre::StringConverter::parseVector3(sv[i]);
        conn.push_back(v);
    }

    sv = cfg.getMultiSetting("front", "connection");
    for(int i=0; i<numwheels; i++)
    {
        bool val = Ogre::StringConverter::parseBool(sv[i]);
        front.push_back(val);
    }

    float mass = Ogre::StringConverter::parseReal(cfg.getSetting("mass", "setup"));
    float friction = Ogre::StringConverter::parseReal(cfg.getSetting("friction", "setup"));
    float suspensionstiffness = Ogre::StringConverter::parseReal(cfg.getSetting("suspensionstiffness", "setup"));
    float wheelsdampingrelaxation = Ogre::StringConverter::parseReal(cfg.getSetting("wheelsdampingrelaxation", "setup"));
    float wheelsdampingcompression = Ogre::StringConverter::parseReal(cfg.getSetting("wheelsdampingcompression", "setup"));
    float frictionslip = Ogre::StringConverter::parseReal(cfg.getSetting("frictionslip", "setup"));
    float rollinfluence = Ogre::StringConverter::parseReal(cfg.getSetting("rollinfluence", "setup"));
    float maxsuspensiontravelcm = Ogre::StringConverter::parseReal(cfg.getSetting("maxsuspensiontravelcm", "setup"));
    float maxsuspensionforce = Ogre::StringConverter::parseReal(cfg.getSetting("maxsuspensionforce", "setup"));
    float wheelradius = Ogre::StringConverter::parseReal(cfg.getSetting("wheelradius", "setup"));
    float suspensionrestlenght = Ogre::StringConverter::parseReal(cfg.getSetting("suspensionrestlenght", "setup"));

    sv = cfg.getMultiSetting("materialcc", "effect");
    svmat = cfg.getMultiSetting("material", "effect");

    Ogre::String vName = "vehicle" + Ogre::StringConverter::toString(id);
    edge::ConvexHullShape* vshape = new edge::ConvexHullShape(vName+"_shape", m_world->getActiveSmgr(), visual, col);
    btVector3 inertia;
    vshape->getPhysics()->setLocalScaling(Util::toBtVector3(colscaling));
    vshape->getPhysics()->calculateLocalInertia(mass, inertia);
    CVehicle* v = new CVehicle(vName, m_world, vshape, mass, inertia);
    v->setType(edge::Body::VEHICLE);
    m_world->addBody(v);
    v->getGraphics()->setScale(scaling);
    v->getPhysics()->setFriction(friction);

    btQuaternion r180;
    r180.setEulerZYX(0,M_PI,0);
    btTransform switcher = btTransform::getIdentity();
    switcher.setRotation(r180);
    for(int i=0; i<conn.size(); i++)
    {
        v->addWheel(wheel, Util::toBtVector3(conn[i]), suspensionrestlenght, wheelradius, front[i]);
        v->getWheelGraphics(i)->getSceneNode()->setScale(wheelscaling);
        if(i%2 == wheelflip)
        v->getWheelGraphics(i)->setLocalTransform(switcher);
    }

        for(int z=0; z<sv.size(); z++)
        for(int i=0; i<v->getShape()->getGraphics()->getNumSubEntities(); i++)
        {
            v->getShape()->getGraphics()->getSubEntity(i)->getMaterial()->setCullingMode(Ogre::CULL_NONE);
            if(v->getShape()->getGraphics()->getSubEntity(i)->getMaterial()->getName() == sv[z])
            {
                //Ogre::MaterialPtr mat = v->getShape()->getGraphics()->getSubEntity(i)->getMaterial()->clone("_iLeather_White"+Ogre::StringConverter::toString(m_vehicleCounter));
                Ogre::MaterialPtr mat;
                if(reflection)
                    mat = Ogre::MaterialManager::getSingleton().getByName(svmat[z]);
                else
                    mat = v->getShape()->getGraphics()->getSubEntity(i)->getMaterial();
                //Ogre::MaterialPtr mat = Ogre::MaterialManager::getSingleton().getByName("Examples/DynamicCubeMap");
                mat = mat->clone(sv[z] + Ogre::StringConverter::toString(id));

                float rr = (rand()%1000)/1000.f;
                float rg = (rand()%1000)/1000.f;
                float rb = (rand()%1000)/1000.f;

                mat->setAmbient(rr, rg, rb);
                mat->setDiffuse(rr, rg, rb, 1);
                mat->setCullingMode(Ogre::CULL_NONE);
                v->getShape()->getGraphics()->getSubEntity(i)->setMaterial(mat);

                //v->getShape()->getGraphics()->getSubEntity(i)->setMaterial(Ogre::MaterialManager::getSingleton().getByName("Ogre/Earring"));
            }
        }

    for (int i=0;i<v->getNumWheels();i++)
    {
        btWheelInfo& wheel = v->getRaycast()->getWheelInfo(i);
        wheel.m_suspensionStiffness = suspensionstiffness;
        wheel.m_wheelsDampingRelaxation = wheelsdampingrelaxation;
        wheel.m_wheelsDampingCompression = wheelsdampingcompression;
        wheel.m_frictionSlip = frictionslip;
        //wheel.m_frictionSlip = 5.5f;
        wheel.m_rollInfluence = rollinfluence;
        wheel.m_maxSuspensionTravelCm = maxsuspensiontravelcm;
        wheel.m_maxSuspensionForce = maxsuspensionforce;
    }

    v->setSpeedInc(speedinc);
    v->setMaxSpeed(maxspeed);
    v->setBrakeVal(brake);
    v->setSteerInc(steerinc);
    v->setMaxSteer(maxsteer);
    v->setHandling(handling);

    id++;

    return v;
}

void Loader::loadVehicleList(Ogre::StringVector& cfgFileList)
{
    Ogre::ConfigFile cf;
    cf.load("media/vehicles/list.ini");

    Ogre::ConfigFile::SectionIterator seci = cf.getSectionIterator();
    while (seci.hasMoreElements())
    {
        //gre::LogManager::getSingleton().logMessage("Start While");
        Ogre::String secName = seci.peekNextKey();
        seci.getNext();
        if(secName.empty())
            continue;
        Ogre::String cfg = cf.getSetting("cfg", secName);
        cfgFileList.push_back("media/vehicles/" + cfg);
        //Ogre::LogManager::getSingleton().logMessage("End While");
    }
}

Track* Loader::loadTrack(edge::World* m_world, const Ogre::String& cfgFile)
{
    Ogre::ConfigFile cf;
    cf.load(cfgFile);
    Ogre::String mesh = cf.getSetting("mesh", "general");
    Ogre::Vector3 scale = Ogre::StringConverter::parseVector3(cf.getSetting("scale", "general"));
    Ogre::StringVector paths = Ogre::StringUtil::split(cf.getSetting("paths", "general"), ",", 128);
    Ogre::Vector3 pos = Ogre::StringConverter::parseVector3(cf.getSetting("pos", "finish"));
    int a = Ogre::StringConverter::parseInt(cf.getSetting("a", "finish"));
    int b = Ogre::StringConverter::parseInt(cf.getSetting("b", "finish"));
    int c = Ogre::StringConverter::parseInt(cf.getSetting("c", "finish"));
    int r = Ogre::StringConverter::parseInt(cf.getSetting("r", "finish"));

    edge::Body* bdy2 = m_world->createBody(mesh,
                                      new edge::BvhTriangleMesh(mesh+"shape",
                                                         m_world->getActiveSmgr(),
                                                         mesh),
                                      0,btVector3(0,0,0));
    bdy2->setScale(Util::toBtVector3(scale));

    Track* trk = new Track(bdy2);

    for(int i=0; i<paths.size(); i++)
        trk->addPath(Loader::loadPath("media/paths/" + paths[i]));

    trk->setFinishLine(pos, a, b, c, r);

    return trk;
}

void Loader::loadTrackList(Ogre::String& cfgFileList)
{

}
