/*
-----------------------------------------------------------------------------
Filename:    TutorialApplication.cpp
-----------------------------------------------------------------------------

This source file is part of the
   ___                 __    __ _ _    _
  /___\__ _ _ __ ___  / / /\ \ (_) | _(_)
 //  // _` | '__/ _ \ \ \/  \/ / | |/ / |
/ \_// (_| | | |  __/  \  /\  /| |   <| |
\___/ \__, |_|  \___|   \/  \/ |_|_|\_\_|
      |___/
      Tutorial Framework
      http://www.ogre3d.org/tikiwiki/
-----------------------------------------------------------------------------
*/
#include "TutorialApplication.h"
#include <OgreTextAreaOverlayElement.h>
#include <OgreFontManager.h>
#include <OgreOverlayContainer.h>
#include <OgreMemorySTLAllocator.h>
#include <string>

//-------------------------------------------------------------------------------------
TutorialApplication::TutorialApplication(void)
{
}
//-------------------------------------------------------------------------------------
TutorialApplication::~TutorialApplication(void)
{
}

void TutorialApplication::playIntro(void)
{
    Ogre::Overlay* ov = Ogre::OverlayManager::getSingleton().getByName("Overlays/IntroBackgnd");
    ov->getChild("IntroBackgndPanel")->setMaterialName("Basic/WhiteNoAlpha");
    ov->show();

    float vH = mWindow->getViewport(0)->getActualHeight() / 2.f;
    float vW = mWindow->getViewport(0)->getActualWidth() / 2.f;

    Ogre::ResourcePtr ptr = Ogre::MaterialManager::getSingleton().getByName("Basic/Logo");
    Ogre::Material* mat = dynamic_cast<Ogre::Material*>(ptr.getPointer());
    Ogre::TextureUnitState* texUS = mat->getTechnique(0)->getPass(0)->getTextureUnitState(0);
    texUS->setTextureName("ogrelogo-small.png", Ogre::TEX_TYPE_2D);
    float logoH = texUS->getTextureDimensions(0).second / ((float)vH);
    float logoW = texUS->getTextureDimensions(0).first / ((float)vW);

    Ogre::Rectangle2D* rect = new Ogre::Rectangle2D(true);
    rect->setBoundingBox(Ogre::AxisAlignedBox(-100000.0f * Ogre::Vector3::UNIT_SCALE, 100000.0f * Ogre::Vector3::UNIT_SCALE));
    rect->setRenderQueueGroupAndPriority(Ogre::RENDER_QUEUE_OVERLAY, 200);
    rect->setCorners(-logoW/2.f, logoH/2.f, logoW/2.f, -logoH/2.f);
    rect->setMaterial("Basic/Logo");

    Ogre::SceneNode* lnode = mSceneMgr->getRootSceneNode()->createChildSceneNode("logonode");
    lnode->attachObject(rect);

    FadeScreenEffect* fse = new FadeScreenEffect(mRoot);
    fse->setSpeed(0.0075f);
    fse->setScreenViewTime(500);
    fse->fade();
    fse->csleep(500);

    texUS->setTextureName("bulletphysics_logo.png", Ogre::TEX_TYPE_2D);
    logoH = texUS->getTextureDimensions(0).second / ((float)vH);
    logoW = texUS->getTextureDimensions(0).first / ((float)vW);

    rect->setCorners(-logoW/2.f, logoH/2.f, logoW/2.f, -logoH/2.f);

    fse->fade();
    fse->csleep(500);

    texUS->setTextureName("gamedeve_logo.png", Ogre::TEX_TYPE_2D);
    logoH = texUS->getTextureDimensions(0).second / ((float)vH);
    logoW = texUS->getTextureDimensions(0).first / ((float)vW);

    rect->setCorners(-logoW/2.f, logoH/2.f, logoW/2.f, -logoH/2.f);

    fse->setSpeed(0.005f);
    fse->setScreenViewTime(2000);
    fse->fade();

    delete fse;
}

//-------------------------------------------------------------------------------------
void TutorialApplication::createScene(void)
{
    /*Ogre::OverlayManager& overlayManager = Ogre::OverlayManager::getSingleton();
    Ogre::OverlayContainer* panel = static_cast<Ogre::OverlayContainer*>(
    overlayManager.createOverlayElement("Panel", "PanelName"));
    panel->setMetricsMode(Ogre::GMM_PIXELS);
    panel->setPosition(0, 0);
    panel->setDimensions(0.5, 0.5);*/

    /*LangConfig config("fr.ini");
    Ogre::FontManager::getSingleton().load( config["font"], "Popular", true );
    Ogre::TextAreaOverlayElement* textArea = static_cast<Ogre::TextAreaOverlayElement*>(
    overlayManager.createOverlayElement("TextArea", "TextAreaName"));
    textArea->setMetricsMode(Ogre::GMM_RELATIVE);
    textArea->setPosition(0.0, 0.0);
    textArea->setDimensions(0.5, 0.2);
    textArea->setCaption(config["loading"]);
    //textArea->setCaption(Ogre::UTFString());
    textArea->setCharHeight(0.1);
    textArea->setFontName(config["font"]);

    //textArea->setFontName("SdkTrays/Value");
    //textArea->setColourBottom(Ogre::ColourValue(0.3, 0.5, 0.3));
    //textArea->setColourTop(Ogre::ColourValue(0.0, 0.7, 0.5));
    textArea->setColour(Ogre::ColourValue(1.0, 1.0, 1.0));

    // Create an overlay, and add the panel
    Ogre::Overlay* overlay = overlayManager.create("OverlayName");
    overlay->add2D(panel);
    overlay->setZOrder(650);

    // Add the text area to the panel
    panel->addChild(textArea);

    // Show the overlay
    overlay->show();*/

    // Play the intro scene first
    playIntro();

    // create your scene here :)
    Ogre::Entity* ent = mSceneMgr->createEntity("e01", "ogrehead.mesh");
    Ogre::SceneNode* node = mSceneMgr->getRootSceneNode()->createChildSceneNode("n01");
    node->attachObject(ent);

    Ogre::Light* l = mSceneMgr->createLight("sun");
    Ogre::SceneNode* node2 = mSceneMgr->getRootSceneNode()->createChildSceneNode("l01");
    node2->attachObject(l);
    node2->setPosition(-100, 100, 0);

    mTrayMgr->getTrayContainer(OgreBites::TL_NONE)->hide();

    mTrayMgr->showLogo(OgreBites::TL_RIGHT);
    mTrayMgr->createSeparator(OgreBites::TL_RIGHT, "LogoSep");
    mTrayMgr->createButton(OgreBites::TL_RIGHT, "StartStop", "Start Sample");
    mTrayMgr->createButton(OgreBites::TL_RIGHT, "UnloadReload", "Unload Samples");
    mTrayMgr->createButton(OgreBites::TL_RIGHT, "Configure", "Configure");
    mTrayMgr->createButton(OgreBites::TL_RIGHT, "Quit", "Quit");


}



#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
    INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
#else
    int main(int argc, char *argv[])
#endif
    {
        // Create application object
        TutorialApplication app;

        try {
            app.go();
        } catch( Ogre::Exception& e ) {
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
            MessageBox( NULL, e.getFullDescription().c_str(), "An exception has occured!", MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else
            std::cerr << "An exception has occured: " <<
                e.getFullDescription().c_str() << std::endl;
#endif
        }

        return 0;
    }

#ifdef __cplusplus
}
#endif
