#include "VehicleAI.h"

VehicleAI::VehicleAI(CVehicle* vehicle,
                     Track* track, edge::ai::Path* path)
: m_v(vehicle), m_p(path), m_csi(0), m_blocked(false), mTrack(track)
{

    Ogre::SceneManager* smgr = m_v->getWorld()->getActiveSmgr();

    m_cs = m_p->getStation(m_csi);
    m_ns = m_p->getStation(m_csi+1);
    btVector3 v = m_p->getStation(0)->getPosition();
    m_erc = btVector3(0, 0, 0);
}

VehicleAI::~VehicleAI()
{

}

void VehicleAI::action()
{
    m_v->setBrake(0, 2);
    m_v->setBrake(0, 3);
    m_v->setBrake(0, 0);
    m_v->setBrake(0, 1);

    float nP2F = mTrack->getPointToFinish(edge::Util::toVector3(m_v->getPosition()));
    float sgn1, sgn2;

    if(m_v->mP2Finish >= 0)
        sgn1 = 1.f;
    else
        sgn1 = -1.f;
    if(nP2F >= 0)
        sgn2 = 1.f;
    else
        sgn2 = -1.f;

    if(sgn1 + sgn2 == 0 && mTrack->testDistance(edge::Util::toVector3(m_v->getPosition())))
    {
        m_v->mLap += sgn2;
    }

    m_v->mP2Finish = nP2F;


    float tdistance = m_cs->getPosition().distance(m_v->getPosition());
    btVector3 twe = m_cs->getPosition() + m_erc;
    btVector3 tdv = twe - m_v->getPhysics()->getCenterOfMassPosition();
    tdv.normalize();
    btVector3 vdv = quatRotate(m_v->getPhysics()->getCenterOfMassTransform().getRotation(), btVector3(1,0,0));
    vdv.normalize();
    //float pdot = vdv.dot(tdv) / (tdistance/10.f) / (m_maxSpeed/m_v->getCurrentSpeedKmh());
    float maxsteer;
    if(m_v->getCurrentSpeedKmh() <= 0)
        maxsteer = m_v->getMaxSteer();
    else
    maxsteer = (m_v->getMaxSpeed() / (m_v->getCurrentSpeedKmh())) / 11.f * m_v->getMaxSteer() * 2.5f;
    float pdot = vdv.dot(tdv) * 0.33f;
    if(m_blocked) pdot *= -3.f;
    /*printf("V=%f\n",va*180/M_PI);
    printf("T=%f\n",ta*180/M_PI);
    printf("DOT=%f\n",adot*180/M_PI);
    printf("STEER=%f\n",er*180/M_PI);
    printf("ID=%i\n", m_csi);
    printf("DIS=%f\n",m_cs->getPosition().distance(m_v->getPhysics()->getCenterOfMassPosition()));*/
    float steering = pdot;
    if(steering > maxsteer*2.f)
        steering = maxsteer;
    if(steering < -maxsteer*2.f)
        steering = -maxsteer;
    m_v->setSteeringValue(steering,0);
    m_v->setSteeringValue(steering,1);

    if(m_v->getCurrentSpeedKmh()>m_v->getMaxSpeed()*1.5f)
    {
        m_v->applyEngineForce(0,0);
        m_v->applyEngineForce(0,1);
    }
    else
    {
        float force = m_v->getSpeedInc() - (m_v->getCurrentSpeedKmh() * m_v->getSpeedInc() / m_v->getMaxSpeed());
        m_v->applyEngineForce(force*1.5f, 0);
        m_v->applyEngineForce(force*1.5f, 1);
    }

    if(tdistance<20.f)
    {
        if(++m_csi >= m_p->getNumStations()) m_csi = 0;
        m_cs = m_p->getStation(m_csi);
        m_ns = m_p->getStation(m_csi+1);
        m_erc = btVector3(rand()%200/100.f-1.f, 0, rand()%200/100.f-1.f);
    }

    if(tdistance<50.f && !m_blocked)
    {
        float ntdistance = m_cs->getPosition().distance(m_ns->getPosition());
        btVector3 ntdv = m_ns->getPosition() - m_cs->getPosition();
        ntdv.normalize();
        float ndot = vdv.dot(ntdv);
        //float ss = (pow(ndot*180/M_PI, 2) / (tdistance*2.f));
        //float brake = (m_v->getCurrentSpeedKmh() - ss) / 25.f;
        float brake = ndot*2.5f;

        m_v->applyEngineForce(0,0);
        m_v->applyEngineForce(0,1);
        m_v->setBrake(brake, 2);
        m_v->setBrake(brake, 3);
        m_v->setBrake(brake/1.5f, 0);
        m_v->setBrake(brake/1.5f, 1);
    }

    if(m_v->getCurrentSpeedKmh() > fabs(2.f) && !m_blocked)
        m_blockTime.reset();

    if(m_blockTime.getTimeMilliseconds() > 5000)
        m_blocked = true;

    if(m_blocked && m_blockTime.getTimeMilliseconds() < 7000)
    {
        m_v->applyEngineForce(-750,0);
        m_v->applyEngineForce(-750,1);
    }
    if(m_blockTime.getTimeMilliseconds() > 9500)
    {
        m_blocked = false;
        m_blockTime.reset();
    }
}
