#include "edge_demo.h"

using namespace edge;
using namespace Ogre;

EdgeDemo::EdgeDemo(void)
: m_vehicleCounter(0), m_vctrl(0), m_vlistIndex(0), m_backCam(0), mGameState(GS_SELECT_LANGUAGE), m_carMirrorRect(0),
  mBlurEffect(true), mReflectionEffect(true), mCurrentLap(1), mMaxLap(4)
{
}

EdgeDemo::~EdgeDemo(void)
{
}

void EdgeDemo::playIntro(void)
{
    Ogre::LogManager::getSingletonPtr()->logMessage("*** Start Intro ***");
    Ogre::Overlay* ov = Ogre::OverlayManager::getSingleton().getByName("Overlays/IntroBackgnd");
    ov->show();
    float vH = mWindow->getViewport(0)->getActualHeight();
    float vW = mWindow->getViewport(0)->getActualWidth();
    Ogre::ResourcePtr ptr = Ogre::MaterialManager::getSingleton().getByName("Basic/Logo");
    Ogre::Material* mat = dynamic_cast<Ogre::Material*>(ptr.getPointer());
    Ogre::TextureUnitState* texUS = mat->getTechnique(0)->getPass(0)->getTextureUnitState(0);
    texUS->setTextureName("ogrelogo-small.png", Ogre::TEX_TYPE_2D);
    float logoH = texUS->getTextureDimensions(0).second / ((float)vH);
    float logoW = texUS->getTextureDimensions(0).first / ((float)vW);
    Ogre::Rectangle2D* rect = new Ogre::Rectangle2D(true);
    rect->setBoundingBox(Ogre::AxisAlignedBox(-100000.0f * Ogre::Vector3::UNIT_SCALE, 100000.0f * Ogre::Vector3::UNIT_SCALE));
    rect->setRenderQueueGroupAndPriority(Ogre::RENDER_QUEUE_OVERLAY, 200);
    rect->setCorners(-logoW/2.f, logoH/2.f, logoW/2.f, -logoH/2.f);
    rect->setMaterial("Basic/Logo");
    Ogre::SceneNode* lnode = mSceneMgr->getRootSceneNode()->createChildSceneNode("logonode");
    lnode->attachObject(rect);
    FadeScreenEffect* fse = new FadeScreenEffect(mRoot);
    fse->setSpeed(0.0075f);
    fse->setScreenViewTime(500);
    fse->fade();
    fse->csleep(500);

    texUS->setTextureName("bulletphysics_logo.png", Ogre::TEX_TYPE_2D);
    logoH = texUS->getTextureDimensions(0).second / ((float)vH);
    logoW = texUS->getTextureDimensions(0).first / ((float)vW);
    rect->setCorners(-logoW/2.f, logoH/2.f, logoW/2.f, -logoH/2.f);
    fse->fade();
    fse->csleep(500);
    texUS->setTextureName("gamedeve_logo.png", Ogre::TEX_TYPE_2D);
    logoH = texUS->getTextureDimensions(0).second / ((float)vH);
    logoW = texUS->getTextureDimensions(0).first / ((float)vW);
    rect->setCorners(-logoW/2.f, logoH/2.f, logoW/2.f, -logoH/2.f);
    fse->setSpeed(0.005f);
    fse->setScreenViewTime(2000);
    fse->fade();
    delete fse;
    mSceneMgr->getRootSceneNode()->removeAndDestroyChild(lnode->getName());
    Ogre::LogManager::getSingletonPtr()->logMessage("*** Finish Intro ***");
}

void EdgeDemo::selectLanguage(void)
{
    Ogre::Overlay* ov = Ogre::OverlayManager::getSingleton().getByName("Overlays/IntroBackgnd");
    ov->show();
    mTrayMgr->showCursor();
    Ogre::LogManager::getSingleton().logMessage("Select Language");
    Ogre::ConfigFile cf;
    cf.load("media/langs/list.ini");
    Ogre::ConfigFile::SectionIterator seci = cf.getSectionIterator();
    while (seci.hasMoreElements())
    {
        Ogre::String secName = seci.peekNextKey();
        seci.getNext();
        if(secName.empty())
            continue;
        bool righttoleft = Ogre::StringConverter::parseBool(cf.getSetting("righttoleft", secName));
        Ogre::DisplayString caption = cf.getSetting("caption", secName);
        if(righttoleft) caption = LangConfig::rightToLeft(caption);
        Ogre::String font = cf.getSetting("font", secName);
        Ogre::FontManager::getSingleton().load(font, "Popular");
        OgreBites::Button* b = mTrayMgr->createButton(OgreBites::TL_CENTER, secName, caption, 256);
        Ogre::TextAreaOverlayElement* bta = _getButtonTextArea(b);
        bta->setFontName(font);
        bta->setCharHeight(20);
    }
    Ogre::LogManager::getSingleton().logMessage("End Select Language");
}

void EdgeDemo::showOptionsMenu(void)
{
    OgreBites::Button* btn;
    btn = mTrayMgr->createButton(OgreBites::TL_BOTTOM, "exittomenu", (*mLang)["back"], 256);
    Ogre::TextAreaOverlayElement* bta = _getButtonTextArea(btn);
    bta->setFontName((*mLang)["font"]);
    bta->setCharHeight(20);

    std::vector<OgreBites::CheckBox*> cb;
    OgreBites::CheckBox* cbptr;
    cbptr = mTrayMgr->createCheckBox(OgreBites::TL_CENTER, "oblur", (*mLang)["blur"], 374);
    cbptr->setChecked(mBlurEffect);
    cb.push_back(cbptr);
    cbptr = mTrayMgr->createCheckBox(OgreBites::TL_CENTER, "oreflection", (*mLang)["reflection"], 374);
    cbptr->setChecked(mReflectionEffect);
    cb.push_back(cbptr);

    std::vector<OgreBites::Label*> l;
    l.push_back(mTrayMgr->createLabel(OgreBites::TL_TOP, "toptions", (*mLang)["options"], 320));

    for(int i=0; i<cb.size(); i++)
    {
        Ogre::TextAreaOverlayElement* bta = _getCheckBoxTextArea(cb[i]);
        bta->setFontName((*mLang)["font"]);
        bta->setCharHeight(20);
    }

    for(int i=0; i<l.size(); i++)
    {
        Ogre::TextAreaOverlayElement* bta = _getLabelTextArea(l[i]);
        bta->setFontName((*mLang)["font"]);
        bta->setCharHeight(20);
    }
}

void EdgeDemo::createGameMenu(void)
{
    Ogre::Overlay* ov = Ogre::OverlayManager::getSingleton().getByName("Overlays/IntroBackgnd");
    ov->show();
    mTrayMgr->showCursor();
    std::vector<OgreBites::Button*> btnList;
    OgreBites::Button* b;
    b = mTrayMgr->createButton(OgreBites::TL_CENTER, "startgame", (*mLang)["startgame"], 256);
    btnList.push_back(b);
    b = mTrayMgr->createButton(OgreBites::TL_CENTER, "options", (*mLang)["options"], 256);
    btnList.push_back(b);
    mTrayMgr->createSeparator(OgreBites::TL_CENTER, "mmsp01", 256);
    b = mTrayMgr->createButton(OgreBites::TL_CENTER, "exit", (*mLang)["exit"], 256);
    btnList.push_back(b);

    Ogre::FontManager::getSingleton().load((*mLang)["font"], "Popular");
    for(int i=0; i<btnList.size(); i++)
    {
        Ogre::TextAreaOverlayElement* bta = _getButtonTextArea(btnList[i]);
        Ogre::LogManager::getSingleton().logMessage("Font="+(*mLang)["font"]);
        bta->setFontName((*mLang)["font"]);
        bta->setCharHeight(20);
    }
}

void EdgeDemo::selectVehicle(void)
{
    Ogre::Overlay* ov = Ogre::OverlayManager::getSingleton().getByName("Overlays/IntroBackgnd");
    ov->show();
    mTrayMgr->showCursor();
    Ogre::Light* light0 = mSceneMgr->createLight("previewlight1");
    light0->setPosition(Ogre::Vector3(-500,500,100));
    light0->setDiffuseColour(0.7,0.7,0.7);
    light0->setType(Ogre::Light::LT_DIRECTIONAL);
    light0->setDirection(1,-1,-0.2);

    Ogre::Light* light1 = mSceneMgr->createLight("previewlight2");
    light1->setPosition(Ogre::Vector3(500,600,100));
    light1->setDiffuseColour(0.7,0.7,0.7);
    light1->setType(Ogre::Light::LT_POINT);

    mCamera->setPosition(10, 7, 10);
    mCamera->lookAt(0,0,0);
    mCurrentCfgFile = 0;

    std::vector<OgreBites::Button*> b;
    b.push_back(mTrayMgr->createButton(OgreBites::TL_RIGHT, "nextcar", (*mLang)["next"], 174));
    b.push_back(mTrayMgr->createButton(OgreBites::TL_LEFT, "prevcar", (*mLang)["prev"], 174));
    b.push_back(mTrayMgr->createButton(OgreBites::TL_BOTTOMRIGHT, "startrace", (*mLang)["startgame"], 174));
    b.push_back(mTrayMgr->createButton(OgreBites::TL_BOTTOMLEFT, "exittomenu", (*mLang)["back"], 174));

    std::vector<OgreBites::Label*> l;
    l.push_back(mTrayMgr->createLabel(OgreBites::TL_TOP, "choosecar", (*mLang)["choosecar"], 320));

    for(int i=0; i<b.size(); i++)
    {
        Ogre::TextAreaOverlayElement* bta = _getButtonTextArea(b[i]);
        bta->setFontName((*mLang)["font"]);
        bta->setCharHeight(20);
    }

    for(int i=0; i<l.size(); i++)
    {
        Ogre::TextAreaOverlayElement* bta = _getLabelTextArea(l[i]);
        bta->setFontName((*mLang)["font"]);
        bta->setCharHeight(20);
    }

    mCurrentVehicle = Loader::loadVehicle(m_world, mCfgFileList[mCurrentCfgFile], m_vehicleCounter, mReflectionEffect);
    mCurrentVehicle->getGraphics()->getAttachedObject(0)->setRenderQueueGroupAndPriority(Ogre::RENDER_QUEUE_OVERLAY, 250);
    for(int i=0; i<mCurrentVehicle->getNumWheels(); i++)
        mCurrentVehicle->getWheelGraphics(i)->getSceneNode()->getAttachedObject(0)->setRenderQueueGroupAndPriority(Ogre::RENDER_QUEUE_OVERLAY, 250);
    mCurrentVehicle->getPhysics()->setAngularVelocity(btVector3(0,1,0));

    m_world->getPhysicsWorld()->setGravity(btVector3(0,0,0));
    m_world->stepSimulation(1.f/60, 10);

    //Ogre::Overlay* ov = Ogre::OverlayManager::getSingleton().getByName("Overlays/IntroBackgnd");
    //ov->add3D(sn);
}

void EdgeDemo::showPauseMenu()
{
    mTrayMgr->showCursor();
    std::vector<OgreBites::Button*> b;
    b.push_back(mTrayMgr->createButton(OgreBites::TL_CENTER, "continue", (*mLang)["continue"], 256));
    b.push_back(mTrayMgr->createButton(OgreBites::TL_CENTER, "exittomenu", (*mLang)["exit"], 256));
    for(int i=0; i<b.size(); i++)
    {
        Ogre::TextAreaOverlayElement* tao = _getButtonTextArea(b[i]);
        tao->setFontName((*mLang)["font"]);
        tao->setCharHeight(20);
    }
}

void EdgeDemo::startGame(void)
{
    Ogre::Overlay* ov = Ogre::OverlayManager::getSingleton().getByName("Overlays/IntroBackgnd");
    ov->hide();
    mTrayMgr->hideCursor();
    Ogre::ColourValue fadeColour(0.9, 0.9, 0.9);
    mWindow->getViewport(0)->setBackgroundColour(fadeColour);
    //mSceneMgr->setFog(Ogre::FOG_LINEAR, fadeColour, 0.0, 50, 800);
    mSceneMgr->setFog(Ogre::FOG_EXP, fadeColour, 0.0021, 50, 3000);

    mSceneMgr->setAmbientLight(Ogre::ColourValue(0.1, 0.1, 0.1));
    mSceneMgr->setSkyBox(true, "Examples/CloudyNoonSkyBox", 600);
    //mSceneMgr->setSkyDome(true, "Examples/CloudySky", 4, 8);
    //mSceneMgr->setShadowTechnique(Ogre::SHADOWTYPE_STENCIL_ADDITIVE);

    Ogre::LogManager::getSingletonPtr()->logMessage("*** Creating Intro ***");

    createMirror();

    Ogre::LogManager::getSingletonPtr()->logMessage("*** Creating Scene ***");

    mTrack = Loader::loadTrack(m_world, "media/tracks/track01.ini");

    btQuaternion q;
    q.setEulerZYX(0,0,0);

    CVehicle* com = Loader::loadVehicle(m_world, "media/vehicles/auditt.cfg", m_vehicleCounter, mReflectionEffect);
    m_vlist.push_back(com);
    com->setRotation(q);
    com->getPhysics()->translate(btVector3(240.1,-29.f,47.0));
    com->setAIBehaviour(new VehicleAI(com, mTrack, mTrack->getPath(0)));
    CVehicle* com2 = Loader::loadVehicle(m_world, "media/vehicles/beetle.cfg", m_vehicleCounter, mReflectionEffect);
    m_vlist.push_back(com2);
    com2->setRotation(q);
    com2->getPhysics()->translate(btVector3(240.8,-29.f,-0.7f));
    com2->setAIBehaviour(new VehicleAI(com2, mTrack, mTrack->getPath(0)));
    CVehicle* com3 = Loader::loadVehicle(m_world, "media/vehicles/golf.cfg", m_vehicleCounter, mReflectionEffect);
    m_vlist.push_back(com3);
    com3->setRotation(q);
    com3->getPhysics()->translate(btVector3(250.1,-29.f,15.1));
    com3->setAIBehaviour(new VehicleAI(com3, mTrack, mTrack->getPath(0)));

    m_player = mCurrentVehicle;
    m_player->getPhysics()->setAngularVelocity(btVector3(0,0,0));
    m_vlist.push_back(m_player);
    m_vctrl = new VehicleController(m_player, mTrack);
    m_player->setRotation(q);
    m_player->getPhysics()->translate(btVector3(249.9,-29.f,47.0));

    mCamera->setPosition(Util::toVector3(m_player->getPosition()+btVector3(0, +5, -15)));
    mCamera->lookAt(Util::toVector3(m_player->getPosition()));

    Ogre::Light* light0 = mSceneMgr->createLight("light");
    light0->setPosition(Ogre::Vector3(-500,500,100));
    light0->setDiffuseColour(0.7,0.7,0.7);
    light0->setType(Ogre::Light::LT_DIRECTIONAL);
    light0->setDirection(1,-1,-0.2);

    Ogre::Light* light1 = mSceneMgr->createLight("light1");
    light1->setPosition(Ogre::Vector3(500,600,100));
    light1->setDiffuseColour(0.7,0.7,0.7);
    light1->setType(Ogre::Light::LT_POINT);

    Ogre::LogManager::getSingletonPtr()->logMessage("*** Scene Created ***");
    //m_player = com3;

    //Ogre::CompositorInstance* ci = CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0), "Sharpen Edges");
    //ci->setEnabled(true);

    std::vector<OgreBites::Label*> l;
    l.push_back(mTrayMgr->createLabel(OgreBites::TL_BOTTOMRIGHT, "speed", (*mLang)["speed"], 142));
    l.push_back(mTrayMgr->createLabel(OgreBites::TL_TOPRIGHT, "laps", (*mLang)["lap"], 142));
    l.push_back(mTrayMgr->createLabel(OgreBites::TL_CENTER, "t123", "", 342));

    for(int i=0; i<l.size(); i++)
    {
        Ogre::TextAreaOverlayElement* bta = _getLabelTextArea(l[i]);
        bta->setFontName((*mLang)["font"]);
        bta->setCharHeight(20);
    }

    t123.reset();
    m_world->stepSimulation(1.f/60, 10);
    mRoot->renderOneFrame();
}

void EdgeDemo::createScene(void)
{
    Ogre::Overlay* ov = Ogre::OverlayManager::getSingleton().getByName("Overlays/IntroBackgnd");
    ov->getChild("IntroBackgndPanel")->setMaterialName("Basic/WhiteNoAlpha");
    srand(time(NULL));
    mCameraMan->setStyle(OgreBites::CS_MANUAL);
    mTrayMgr->hideFrameStats();
    mTrayMgr->hideLogo();
    mTrayMgr->showCursor();

    Ogre::ConfigFile cf;
    cf.load("media/vehicles/list.ini");

    Ogre::ConfigFile::SectionIterator seci = cf.getSectionIterator();
    while (seci.hasMoreElements())
    {
        Ogre::LogManager::getSingleton().logMessage("Start While");
        Ogre::String secName = seci.peekNextKey();
        seci.getNext();
        if(secName.empty())
            continue;
        Ogre::String cfg = cf.getSetting("cfg", secName);
        mCfgFileList.push_back("media/vehicles/" + cfg);
        Ogre::LogManager::getSingleton().logMessage("End While");
    }

    if(mBlurEffect)
    {
        mRadialBlur = CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0), "Radial Blur");
        if(mRadialBlur)
        mRadialBlur->setEnabled(false);
    }

    playIntro();

    selectLanguage();
}

void EdgeDemo::createMirror(void)
{
    if(!m_backCam)
    {
        m_backCam = mSceneMgr->createCamera("backCam");
        float x,y,z,w;
        m_backCam->setNearClipDistance(0.01f);
        m_backCam->getFrustumExtents(x,y,z,w);
        m_backCam->setFrustumExtents(x,y,z/4.f,w/4.f);
    }
    Ogre::TexturePtr rtt_texture = Ogre::TextureManager::getSingleton().createManual("CarMirrorTex", Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, Ogre::TEX_TYPE_2D, mWindow->getWidth(), mWindow->getHeight(), 0, Ogre::PF_R8G8B8, Ogre::TU_RENDERTARGET);
    Ogre::RenderTexture *renderTexture = rtt_texture->getBuffer()->getRenderTarget();

    //renderTexture->addViewport(mCamera);
    renderTexture->addViewport(m_backCam);
    //mCamera->setFrustumExtents(-1,1,-1,1);
    renderTexture->getViewport(0)->setClearEveryFrame(true);
    renderTexture->getViewport(0)->setBackgroundColour(Ogre::ColourValue::Black);
    renderTexture->getViewport(0)->setOverlaysEnabled(false);
    renderTexture->setAutoUpdated(true);
    renderTexture->addListener(this);

    if(!m_carMirrorRect)
    m_carMirrorRect = new Ogre::Rectangle2D(true);
    m_carMirrorRect->setCorners(-0.4f, 0.95f, 0.4f, 0.7f);
    m_carMirrorRect->setBoundingBox(Ogre::AxisAlignedBox(-100000.0f * Ogre::Vector3::UNIT_SCALE, 100000.0f * Ogre::Vector3::UNIT_SCALE));

    Ogre::SceneNode* carMirrorNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("CarMirror");
    carMirrorNode->attachObject(m_carMirrorRect);

    Ogre::MaterialPtr renderMaterial = Ogre::MaterialManager::getSingleton().create("CarMirrorMat", Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);
    Ogre::Technique* matTechnique = renderMaterial->createTechnique();
    matTechnique->createPass();
    renderMaterial->getTechnique(0)->getPass(0)->setLightingEnabled(false);
    renderMaterial->getTechnique(0)->getPass(0)->createTextureUnitState("CarMirrorTex");

    m_carMirrorRect->setMaterial("CarMirrorMat");
}

void EdgeDemo::destroyGameScene()
{
    delete m_carMirrorRect;
    m_carMirrorRect = 0;
    Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
    Ogre::TextureManager::getSingleton().removeAll();
    for(int i=0; i<m_vlist.size(); i++)
        //m_world->destroyBody(m_vlist[i]->getName());
        delete m_vlist[i];
    m_vlist.clear();
    m_vehicleCounter = 0;
    Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
    //mSceneMgr->destroyAllCameras();
    //mSceneMgr->destroyCamera("backCam");
    //mSceneMgr->getRootSceneNode()->removeAndDestroyChild("CarMirror");
    //mSceneMgr->getRootSceneNode()->removeAndDestroyAllChildren();
    mSceneMgr->clearScene();
    Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
    //mSceneMgr->getRootSceneNode()->removeAndDestroyAllChildren();
    Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
    mCurrentCfgFile = 0;
    Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
    //mSceneMgr->destroyAllEntities();
    Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
    for(int i=0; i<m_world->getPhysicsWorld()->getNumCollisionObjects(); i++)
        m_world->getPhysicsWorld()->removeCollisionObject(m_world->getPhysicsWorld()->getCollisionObjectArray()[i]);
    Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
    Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
}

bool EdgeDemo::frameRenderingQueued(const Ogre::FrameEvent& evt)
{
    if(mWindow->isClosed())
        return false;

    if(mShutDown)
        return false;

    //Need to capture/update each device
    mKeyboard->capture();
    mMouse->capture();
    mTrayMgr->adjustTrays();
    mTrayMgr->frameRenderingQueued(evt);

    //guiUpdate();


    if(mGameState == GS_SELECT_LANGUAGE)
    {

    }

    if(mGameState == GS_CHOOSE_VEHICLE)
    {
        m_world->stepSimulation(1.f/60, 10);
    }

    if(mGameState == GS_MAIN_MENU)
    {

    }

    if(mGameState == GS_OPTIONS)
    {

    }

    if(mGameState == GS_PREPLAYING)
    {
        OgreBites::Label* l = static_cast<OgreBites::Label*>(mTrayMgr->getWidget("t123"));
        int tb = 1000;
        if(t123.getMilliseconds() < tb)
            l->setCaption("1");
        else if(t123.getMilliseconds() < tb*2)
            l->setCaption("2");
        else if(t123.getMilliseconds() < tb*3)
            l->setCaption("3");
        else if(t123.getMilliseconds() < tb*3.5)
            l->setCaption((*mLang)["go"]);
        else
        {
            mGameState = GS_PLAYING;
            mTrayMgr->destroyAllWidgetsInTray(OgreBites::TL_CENTER);
        }
        m_stepTimer.reset();
    }

    if(mGameState == GS_PLAYING)
    {
        if(m_player->getCurrentSpeedKmh()>210 && mBlurEffect)
        {
            if(mRadialBlur)
            mRadialBlur->setEnabled(true);
        }
        else
        {
            if(mRadialBlur)
            mRadialBlur->setEnabled(false);
        }

        if (!mTrayMgr->isDialogVisible())
        {
            if(mCameraMan)
            mCameraMan->frameRenderingQueued(evt);   // if dialog isn't up, then update the camera
        }

        m_world->stepSimulation(m_stepTimer.getTimeMilliseconds(), 1);
        m_stepTimer.reset();
        m_vctrl->update();

        btVector3 m_cameraPosition = Util::toBtVector3(mCamera->getPosition());
        btTransform chassisWorldTrans = m_player->getPhysics()->getCenterOfMassTransform();
        btVector3 m_cameraTargetPosition = chassisWorldTrans.getOrigin();

        btScalar m_cameraHeight = 9.f;
        btScalar m_minCameraDistance = 7.f;
        btScalar m_maxCameraDistance = 11.f;
        float correctionFactor = 0.0f;

        m_cameraPosition[1] = (15.0*m_cameraPosition[1] + m_cameraTargetPosition[1] + m_cameraHeight)/16.0;

        btVector3 camToObject = m_cameraTargetPosition - m_cameraPosition;

        float cameraDistance = camToObject.length();
        if (cameraDistance < m_minCameraDistance)
        {
            correctionFactor = 0.15*(m_minCameraDistance-cameraDistance)/cameraDistance;
        }
        if (cameraDistance > m_maxCameraDistance)
        {
            correctionFactor = 0.15*(m_maxCameraDistance-cameraDistance)/cameraDistance;
        }
        m_cameraPosition -= correctionFactor*camToObject;
        mCamera->setPosition(Util::toVector3(m_cameraPosition));
        m_cameraTargetPosition[1] += 2.f;
        mCamera->lookAt(Util::toVector3(m_cameraTargetPosition));

        btVector3 camPos = m_player->getPosition();
        btQuaternion camRot = m_player->getPhysics()->getCenterOfMassTransform().getRotation();
        btVector3 mrg = quatRotate(camRot, btVector3(0,0,-5.f));
        camPos += mrg;
        camPos[1] += 1;
        m_backCam->setPosition(Util::toVector3(camPos));
        m_backCam->setOrientation(Util::toQuaternion(camRot));

        OgreBites::Label* l = static_cast<OgreBites::Label*>(mTrayMgr->getWidget("speed"));
        l->setCaption(Ogre::StringConverter::toString(((int)m_player->getCurrentSpeedKmh())) + "  " + (*mLang)["speed"]);
        l = static_cast<OgreBites::Label*>(mTrayMgr->getWidget("laps"));
        int lap = m_player->mLap;
        if(lap < 1) lap = 1;
        l->setCaption(Ogre::StringConverter::toString(lap) + "/" + Ogre::StringConverter::toString(mMaxLap) + " " + (*mLang)["lap"]);

        if(m_player->mLap == mMaxLap)
        {
            mTrayMgr->showCursor();
            OgreBites::Label* l = mTrayMgr->createLabel(OgreBites::TL_CENTER, "end", (*mLang)["alphaversion"], 512);
            OgreBites::Button* b = mTrayMgr->createButton(OgreBites::TL_CENTER, "endok", (*mLang)["back"], 256);
            Ogre::TextAreaOverlayElement* bta = _getButtonTextArea(b);
            bta->setFontName((*mLang)["font"]);
            bta->setCharHeight(20);
            bta = _getLabelTextArea(l);
            bta->setFontName((*mLang)["font"]);
            bta->setCharHeight(20);
            mGameState = GS_END;
        }
    }

    return true;
}

bool EdgeDemo::userFrameRenderingQueued(const Ogre::FrameEvent& evt)
{
    return true;
}

void EdgeDemo::preRenderTargetUpdate(const Ogre::RenderTargetEvent& evt)
{
    if(m_carMirrorRect)
	m_carMirrorRect->setVisible(false);
}

void EdgeDemo::postRenderTargetUpdate(const Ogre::RenderTargetEvent& evt)
{
    if(m_carMirrorRect)
	m_carMirrorRect->setVisible(true);
}

void EdgeDemo::guiUpdate(void)
{
    //mGuiSpeed->setCaption(Ogre::StringConverter::toString((int)m_player->getCurrentSpeedKmh())+" kmh");
    //mGuiLap->setCaption("LAP 1/4");
}

bool EdgeDemo::keyPressed(const OIS::KeyEvent &arg)
{
    //if (mTrayMgr->isDialogVisible()) return true;

    if(arg.key == OIS::KC_C)
    {
        m_player = m_vlist[m_vlistIndex];
        if(++m_vlistIndex >= m_vlist.size())
            m_vlistIndex = 0;
    }
    if(arg.key == OIS::KC_A)
        p.push_back(m_player->getPhysics()->getCenterOfMassTransform().getOrigin());
    if(arg.key == OIS::KC_S)
    {
        FILE* f = fopen("path.bin", "wb+");
        if(!f)
        {
            printf("ERROR FILE\n");
            return false;
        }
        int count = p.size();
        fwrite(&count,sizeof(int),1,f);
        printf("CNT=%i\n", count);
        for(int i=0; i<count; i++)
        {
            fwrite(&p[i][0],sizeof(float),1,f);
            fwrite(&p[i][1],sizeof(float),1,f);
            fwrite(&p[i][2],sizeof(float),1,f);
        }
        fclose(f);
    }

    if(arg.key == OIS::KC_U)
    {
        btVector3 ppos = m_player->getPosition();
        Ogre::LogManager::getSingleton().logMessage("POS = "+Ogre::StringConverter::toString(Util::toVector3(ppos)));
        btVector3 v = quatRotate(m_player->getRotation(), btVector3(1,0,0));
        Ogre::LogManager::getSingleton().logMessage("V = "+Ogre::StringConverter::toString(Util::toVector3(v)));
        float c = -v.x() * ppos.x() -v.z() * ppos.z();
        Ogre::LogManager::getSingleton().logMessage("C = "+Ogre::StringConverter::toString(c));
    }
    if(arg.key == OIS::KC_ESCAPE && mGameState == GS_PLAYING)
    {
        showPauseMenu();
        mGameState = GS_PAUSE;
    }
    else if(arg.key == OIS::KC_RETURN && mGameState == GS_PLAYING)
    {
        btVector3 vpos = m_player->getPosition();
        float distance = 99999999.9f;
        int id=0;
        for(int i=0; i<mTrack->getPath(0)->getNumStations(); i++)
        {
            float cdistance = vpos.distance(mTrack->getPath(0)->getStation(i)->getPosition());
            if(cdistance < distance)
            {
                distance = cdistance;
                id = i;
            }
        }
        int ns;
        if(id+1 >= mTrack->getPath(0)->getNumStations())
            ns = 0;
        else
            ns = id+1;
        btVector3 v2ns = mTrack->getPath(0)->getStation(ns)->getPosition() - vpos;
        v2ns = v2ns.normalized();
        float rot = atan2f(v2ns.z(), v2ns.x());
        m_player->setPosition(mTrack->getPath(0)->getStation(id)->getPosition());
        m_player->getPhysics()->setLinearVelocity(btVector3(0,0,0));
        m_player->getPhysics()->setAngularVelocity(btVector3(0,0,0));
        btQuaternion q;
        q.setEulerZYX(0,rot,0);
        m_player->setRotation(q);
    }

    if(m_vctrl) m_vctrl->keyPressed(arg);
    return true;
}

bool EdgeDemo::userKeyReleased(const OIS::KeyEvent &arg)
{
    if(m_vctrl) m_vctrl->keyReleased(arg);
    return true;
}

void EdgeDemo::buttonHit(OgreBites::Button* button)
{
    if(button->getName() == "AR" || button->getName() == "EN" || button->getName() == "FR")
    {
        Ogre::String langFile("media/langs/");
        Ogre::String bname = button->getName();
        Ogre::StringUtil::toLowerCase(bname);
        langFile += bname;
        langFile += ".ini";
        mLang = new LangConfig(langFile);
        //Ogre::OverlayManager::getSingleton().destroy("Overlays/IntroBackgnd");
        //mTrayMgr->clearAllTrays();
        mTrayMgr->destroyAllWidgets();
        createGameMenu();
        mGameState = GS_MAIN_MENU;
    }
    else if(button->getName() == "startgame")
    {
        mTrayMgr->destroyAllWidgets();
        Ogre::LogManager::getSingletonPtr()->logMessage("*** Start Game Paused ***");
        selectVehicle();
        Ogre::LogManager::getSingletonPtr()->logMessage("*** Finish Game Paused ***");
        mGameState = GS_CHOOSE_VEHICLE;
    }
    else if(button->getName() == "options")
    {
        mTrayMgr->destroyAllWidgets();
        showOptionsMenu();
    }
    else if(button->getName() == "nextcar")
    {
        if(++mCurrentCfgFile >= mCfgFileList.size())
            mCurrentCfgFile = 0;
        Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
        delete mCurrentVehicle;
        //m_world->destroyBody(mCurrentVehicle->getName());
        Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
        mCurrentVehicle = Loader::loadVehicle(m_world, mCfgFileList[mCurrentCfgFile], m_vehicleCounter, mReflectionEffect);
        Ogre::LogManager::getSingleton().logMessage("LINE "+Ogre::StringConverter::toString(__LINE__));
        mCurrentVehicle->getPhysics()->setAngularVelocity(btVector3(0,1,0));
        mCurrentVehicle->getGraphics()->getAttachedObject(0)->setRenderQueueGroupAndPriority(Ogre::RENDER_QUEUE_OVERLAY, 250);
        for(int i=0; i<mCurrentVehicle->getNumWheels(); i++)
            mCurrentVehicle->getWheelGraphics(i)->getSceneNode()->getAttachedObject(0)->setRenderQueueGroupAndPriority(Ogre::RENDER_QUEUE_OVERLAY, 250);
    }
    else if(button->getName() == "prevcar")
    {
        if(--mCurrentCfgFile < 0)
            mCurrentCfgFile = mCfgFileList.size() - 1;
        //delete mCurrentVehicle;
        m_world->destroyBody(mCurrentVehicle->getName());
        mCurrentVehicle = Loader::loadVehicle(m_world, mCfgFileList[mCurrentCfgFile], m_vehicleCounter, mReflectionEffect);
        mCurrentVehicle->getPhysics()->setAngularVelocity(btVector3(0,1,0));
        mCurrentVehicle->getGraphics()->getAttachedObject(0)->setRenderQueueGroupAndPriority(Ogre::RENDER_QUEUE_OVERLAY, 250);
        for(int i=0; i<mCurrentVehicle->getNumWheels(); i++)
            mCurrentVehicle->getWheelGraphics(i)->getSceneNode()->getAttachedObject(0)->setRenderQueueGroupAndPriority(Ogre::RENDER_QUEUE_OVERLAY, 250);
    }
    else if(button->getName() == "continue")
    {
        mTrayMgr->hideCursor();
        mTrayMgr->destroyAllWidgetsInTray(OgreBites::TL_CENTER);
        mGameState = GS_PLAYING;
    }
    else if(button->getName() == "exittomenu")
    {
        destroyGameScene();
        mTrayMgr->destroyAllWidgets();
        createGameMenu();
        mGameState = GS_MAIN_MENU;
    }
    else if(button->getName() == "startrace")
    {
        mSceneMgr->destroyAllLights();
        //mSceneMgr->getRootSceneNode()->removeAllChildren();
        m_world->getPhysicsWorld()->setGravity(btVector3(0, -10, 0));
        mTrayMgr->destroyAllWidgets();
        mGameState = GS_PREPLAYING;
        //mTrayMgr->showFrameStats(OgreBites::TL_BOTTOMLEFT);
        startGame();
    }
    else if(button->getName() == "exit")
    {
        mShutDown = true;
    }
    else if(button->getName() == "endok")
    {
        destroyGameScene();
        mTrayMgr->destroyAllWidgets();
        createGameMenu();
        mTrayMgr->showCursor();
        mGameState = GS_MAIN_MENU;
    }
}

void EdgeDemo::checkBoxToggled(OgreBites::CheckBox* box)
{
    if(box->getName() == "oblur")
    {
        mBlurEffect = box->isChecked();
    }
    else if(box->getName() == "oreflection")
    {
        mReflectionEffect = box->isChecked();
    }
}

Ogre::TextAreaOverlayElement* EdgeDemo::_getButtonTextArea(OgreBites::Button* widget)
{
    Ogre::BorderPanelOverlayElement* bpo = static_cast<Ogre::BorderPanelOverlayElement*>(widget->getOverlayElement());
    Ogre::TextAreaOverlayElement* tao = static_cast<Ogre::TextAreaOverlayElement*>(bpo->getChild(bpo->getName() + "/ButtonCaption"));
    return tao;
}

Ogre::TextAreaOverlayElement* EdgeDemo::_getLabelTextArea(OgreBites::Label* widget)
{
    Ogre::BorderPanelOverlayElement* bpo = static_cast<Ogre::BorderPanelOverlayElement*>(widget->getOverlayElement());
    Ogre::TextAreaOverlayElement* tao = static_cast<Ogre::TextAreaOverlayElement*>(bpo->getChild(bpo->getName() + "/LabelCaption"));
    return tao;
}

Ogre::TextAreaOverlayElement* EdgeDemo::_getCheckBoxTextArea(OgreBites::CheckBox* widget)
{
    Ogre::BorderPanelOverlayElement* bpo = static_cast<Ogre::BorderPanelOverlayElement*>(widget->getOverlayElement());
    Ogre::TextAreaOverlayElement* tao = static_cast<Ogre::TextAreaOverlayElement*>(bpo->getChild(bpo->getName() + "/CheckBoxCaption"));
    return tao;
}

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
    INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
#else
    int main(int argc, char *argv[])
#endif
    {
        // Create application object
        EdgeDemo app;

        try {
            app.go();
        } catch( Ogre::Exception& e ) {
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
            MessageBox( NULL, e.getFullDescription().c_str(), "An exception has occured!", MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else
            std::cerr << "An exception has occured: " <<
                e.getFullDescription().c_str() << std::endl;
#endif
        }

        return 0;
    }

#ifdef __cplusplus
}
#endif
